'****************************************************************************
'
' PBMAIL Paket : GLOBALQP.INC (Base64 and Iso8859-1 Routines)
' Version      : 0.14g
' Compilation  : Thomas Gohel (gohel@basicguru.com)
' Last Files at: http://www.gohel.de
' Last Updates : see directly in the sources
'
' based on UKAW/UKAD technology
'
'****************************************************************************


'-------------------------------------------------------------------------------------------
' Base64 - String decodieren
'-------------------------------------------------------------------------------------------
FUNCTION Get_Base64_Decode(szEncoded AS STRING, iEndOfText AS INTEGER) AS STRING

        DIM icChopMask  AS LOCAL INTEGER  ' Constant 8-bit mask (Faster than using string constants)
        DIM icBitShift  AS LOCAL INTEGER  ' Constant shift mask (Faster than using string constants)
        DIM icStartMask AS LOCAL INTEGER  ' Initial mask value  (Faster than using string constants)
        DIM iRollOver   AS LOCAL INTEGER  ' Decoded Roll over value
        DIM iHighMask   AS LOCAL INTEGER  ' Mask high bits of each char
        DIM iShift      AS LOCAL INTEGER  ' Multiplier shift value
        DIM iLowShift   AS LOCAL INTEGER  ' Mask low bits of each char
        DIM szAlphabet  AS LOCAL STRING   ' Decode/Encode Lookup Table
        DIM szTemp      AS LOCAL STRING   ' Working string
        DIM iPtr        AS LOCAL INTEGER
        DIM iChar       AS LOCAL INTEGER
        DIM iCounter    AS LOCAL INTEGER

        szAlphabet  = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
        icChopMask  = 255
        icBitShift  = 4
        icStartMask = &H10
        iShift      = icBitShift
        iLowShift   = 0
        iRollOver   = 0
        iHighMask   = -1

        ' Check if empty decoded string.
        ' If Empty, return NUL
        IF LEN(szEncoded) = 0 THEN
            FUNCTION = ""
        ELSE
            szTemp = ""
            FOR iCounter = 1 TO LEN(szEncoded)
                iChar = ASC(MID$(szEncoded, iCounter, 1)) ' Get next alphabet
                iPtr = INSTR(szAlphabet, CHR$(iChar)) - 1 ' Get Decoded value
                IF iPtr >= 0 THEN                         ' Check if character is valid
                    IF iShift = icBitShift THEN           ' Char is valid, process it
                        iRollOver = (iPtr * iShift) AND icChopMask ' 1st char in block of 4, keep high part of character
                        ' Reset masks for next character
                        iHighMask = &H30
                        iLowShift = icStartMask
                        iShift = icStartMask
                    ELSE
                        ' Start saving decoded character
                        szTemp = szTemp + CHR$(iRollOver OR ((iPtr AND iHighMask) / iLowShift))
                        ' Calculate next mask and shift values
                        iRollOver = (iPtr * iShift) AND icChopMask
                        iShift = iShift * icBitShift
                        iHighMask = (iHighMask \ icBitShift) OR &H30
                        iLowShift = iLowShift / icBitShift
                        IF iShift > 256 THEN
                            iShift = icBitShift
                            iLowShift = 0
                        END IF
                    END IF
                END IF
            NEXT Counter
            ' Concat last character if required
            IF (iShift > icBitShift AND iShift < 256) THEN
                ' Character remaining in    iRollOver
                IF iEndOfText THEN
                    ' Last string to decode in file
                    szTemp = szTemp + CHR$(iRollOver)
                END IF
            END IF
            FUNCTION = szTemp
        END IF
END FUNCTION

'-------------------------------------------------------------------------------------------
' Base64 - String codieren
'-------------------------------------------------------------------------------------------
FUNCTION Get_Base64_Encode(szUnEncoded AS STRING) AS STRING

        DIM icChopMask  AS LOCAL INTEGER  ' Constant 8-bit mask (Faster than using string constants)
        DIM icBitShift  AS LOCAL INTEGER  ' Constant shift mask (Faster than using string constants)
        DIM icStartMask AS LOCAL INTEGER  ' Initial mask value  (Faster than using string constants)
        DIM iRollOver   AS LOCAL INTEGER  ' Decoded Roll over value
        DIM iHighMask   AS LOCAL INTEGER  ' Mask high bits of each char
        DIM iShift      AS LOCAL INTEGER  ' Multiplier shift value
        DIM iLowShift   AS LOCAL INTEGER  ' Mask low bits of each char
        DIM szAlphabet  AS LOCAL STRING   ' Decode/Encode Lookup Table
        DIM szTemp      AS LOCAL STRING   ' Working string
        DIM iPtr        AS LOCAL INTEGER
        DIM iChar       AS LOCAL INTEGER
        DIM iCounter    AS LOCAL INTEGER
        DIM icLowFill   AS LOCAL INTEGER
        DIM iLowMask    AS LOCAL INTEGER

        ' If Empty, return NUL
        IF LEN(szUnEncoded) = 0 THEN
            FUNCTION = ""
        ELSE
            ' Initialize lookup dictionary and constants
            szAlphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
            icBitShift = 4
            icChopMask = 255
            icLowFill  = 3
            szTemp     = ""    ' Initialize Masks
            iHighMask  = &HFC
            iLowMask   = &H3
            iShift     = &H10
            iRollOver  = 0

            ' Begin Encoding process
            FOR iCounter = 1 TO LEN(szUnEncoded)
                iChar = ASC(MID$(szUnEncoded, iCounter, 1))' Fetch ascii character in decoded string
                ' Calculate Alphabet lookup pointer
                iPtr = ((iChar AND iHighMask) \ (iLowMask + 1)) OR iRollOver
                ' Roll bit patterns
                iRollOver = (iChar AND iLowMask) * iShift
                ' Concatenate encoded character to working encoded string
                szTemp = szTemp + MID$(szAlphabet, iPtr + 1, 1)
                ' Adjust masks
                iHighMask = (iHighMask * icBitShift) AND icChopMask
                iLowMask = iLowMask * icBitShift + icLowFill
                iShift = iShift \ icBitShift
                ' If last character in block, concat last RollOver and
                '   reset masks
                IF iHighMask = 0 THEN
                    szTemp = szTemp + MID$(szAlphabet, iRollOver + 1, 1)
                    iRollOver = 0
                    iHighMask = &HFC
                    iLowMask  = &H3
                    iShift    = &H10
                END IF
            NEXT iCounter

            ' If RollOver remains, concat it to the working string
            IF iShift < &H10 THEN
                szTemp = szTemp + MID$(szAlphabet, iRollOver + 1, 1)
            END IF
            ' Pad encoded string with required '=' pad characters
            iPtr = (LEN(szTemp) MOD 4)
            IF iPtr THEN
                szTemp = szTemp + STRING$(4 - iPtr, "=")
            END IF
            FUNCTION = szTemp
        END IF
END FUNCTION

'-------------------------------------------------------------------------------------------
' Quoted-Printable Text in Plain Text konvertieren (fr Bildausgabe und CrossSecretary)
' bzw. Decode ISO8859-1
'-------------------------------------------------------------------------------------------
FUNCTION Get_Qp2Text(BYVAL EncodedText AS STRING) AS STRING

        DIM i            AS LOCAL LONG        '
        DIM p1           AS LOCAL LONG        '
        DIM p2           AS LOCAL LONG        '
        DIM pChar        AS LOCAL LONG        '
        DIM strText      AS LOCAL STRING      '
        DIM szDest       AS LOCAL ASCIIZ * 210'
        DIM strRun       AS LOCAL STRING      '
        DIM strTmp       AS LOCAL STRING
        DIM Charset      AS LOCAL STRING      '
        DIM Charset_IDs   AS LOCAL STRING     '

        Charset_IDs$ = UCASE$("=?iso-8859-1? =?iso-8859-15? =?windows-1252?")

        ' originale Zeile erstmal umkopieren
        strText = EncodedText

        FOR i& = 1 TO PARSECOUNT(Charset_IDs, " ")

            Charset$ = PARSE$(CharSet_IDs, " ", i&)
            p1 = INSTR(UCASE$(EncodedText), Charset)

            IF p1 > 0 THEN
                ' originale Zeile lschen
                strText = ""
                ' fhrende unkodierte Zeichen sichern
                strRun = LEFT$(EncodedText, p1-1)
                p1 = p1 + LEN(Charset)
                IF INSTR(UCASE$(EncodedText), "?Q?") > 0 THEN
                    ' Quoted Printable:
                    p1 = INSTR(p1 - 1, UCASE$(EncodedText), "?Q?") + 3
                    p2 = INSTR(p1, EncodedText, "?=") - 1

                    FOR pChar = p1 TO p2
                        SELECT CASE MID$(EncodedText, pChar, 1)
                            CASE "="
                                strText = strText & CHR$(VAL("&H" & MID$(EncodedText, pChar + 1, 2)))
                                pChar = pChar + 2
                            CASE "_"
                                strText = strText & CHR$(32)
                            CASE ELSE
                                strText = strText & MID$(EncodedText, pChar, 1)
                        END SELECT
                    NEXT pChar
                    strText = strText & MID$(EncodedText, p2 + 3)
                ELSEIF INSTR(UCASE$(EncodedText), "?B?") > 0 THEN
                    ' Base64
                    p1 = INSTR(p1 - 1, UCASE$(EncodedText), "?B?") + 3
                    p2 = INSTR(p1, UCASE$(EncodedText), "?=") - 1
                    FOR pChar = p1 TO p2
                        SELECT CASE MID$(EncodedText, pChar, 1)
                            CASE "="
                                'strText = strText & CHR$(VAL("&H" & MID$(EncodedText, pChar + 1, 2)))
                                strText = Get_Base64_Decode(strText, 0)
                                pChar = pChar + 2
                            CASE "_"
                                strText = strText & CHR$(32)
                            CASE ELSE
                                strText = strText & MID$(EncodedText, pChar, 1)
                        END SELECT
                    NEXT pChar
                    strText = strText & MID$(EncodedText, p2 + 3)
                ELSE
                    ' keine bekannte Variante gefunden
                    strText = EncodedText
                END IF
                strText = strRun + strText
            ELSE
            END IF
        NEXT i&

        ' Zeichensatz auf Console umstricken (blockweise da Zeile sich eventuell nicht
        ' an RFC-Standard hlt)
        ' Hinweis: Es werden auch nicht-kodierte Zeilen per Default umgewandelt, da diverse
        '          Mail/Newsreader hierbei die Kodierung vergessen
        strRun       = ""
        FOR p1 = 1 TO LEN(strText) STEP 200
            strTmp = MID$(strText, p1, 200)
            CharToOem strTmp + CHR$(0), szDest
            strRun = strRun + szDest
        NEXT p1

        FUNCTION = strRun

END FUNCTION
